﻿Option Infer On
Option Strict On
Public Class Form1
    Private ReadOnly m_PFN As String = "C:\TestDir\MyBinfile"

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        TextBox1.Font = New Font("Courier New", 10)
        TextBox2.Font = New Font("Courier New", 10)
        Dim Doc1 = New Document(m_PFN & "1.bin", Endianness.IntelLittleEndian, Data.DefaultData)
        Dim Doc2 = New Document(m_PFN & "2.bin", Endianness.MotorolaBigEndian, Data.DefaultData)
        Doc1.Write()
        Doc2.Write()
        UpdateView()
    End Sub
    Sub UpdateView()
        TextBox1.Text = Document.ReadFileToStr(m_PFN & "1.bin")
        TextBox2.Text = Document.ReadFileToStr(m_PFN & "2.bin")
    End Sub
    Private Sub Button1_Click(sender As Object, e As EventArgs) Handles Button1.Click
        Dim Doc1 = New Document(m_PFN & "1.bin", Endianness.IntelLittleEndian, New Data)
        Dim Doc2 = New Document(m_PFN & "2.bin", Endianness.MotorolaBigEndian, New Data)
        Doc1.Read()
        Doc2.Read()
        MsgBox(Doc1.Data.IsEqual(Doc2.Data))
    End Sub
End Class
Public Enum Endianness As UInt16
    IntelLittleEndian = &H4949 ' = "II"
    MotorolaBigEndian = &H4D4D ' = "MM"
End Enum

Public Class Document
    Private m_pfn As String
    Private m_en As Endianness
    Public Data As Data
    Public Sub New(aPFN As String, en As Endianness, aData As Data)
        m_pfn = aPFN : m_en = en : Data = aData
    End Sub
    Public Shared Function ReadFileToStr(aPFN As String) As String
        Dim fs As New IO.FileStream(aPFN, IO.FileMode.Open)
        Dim b(CInt(fs.Length) - 1) As Byte
        fs.Read(b, 0, CInt(fs.Length))
        Dim sb As New Text.StringBuilder
        For i = 0 To b.Length - 1
            sb.Append(b(i).ToString("x2"))
            If i < b.Length - 1 Then sb.Append(" ")
            If (0 < i) And (((i + 1) Mod 16) = 0) Then sb.Append(vbNewLine)
        Next
        fs.Close()
        Return sb.ToString
    End Function
    Public Function IsEqual(other As Document) As Boolean
        IsEqual = Data.IsEqual(other.Data)
    End Function
    Public Sub Write()
        Dim fs As New IO.FileStream(m_pfn, IO.FileMode.OpenOrCreate)
        Dim bw As New IO.BinaryWriter(fs)
        bw.Write(m_en)
        Dim ebw As New EndianBinaryWriter(bw)
        bw = EndianBinaryWriter.DecideEndianWriter(ebw, m_en)
        Data.Write(bw)
        fs.Close()
    End Sub
    Public Sub Read()
        Dim fs As New IO.FileStream(m_pfn, IO.FileMode.OpenOrCreate)
        Dim br As New IO.BinaryReader(fs)
        m_en = Endianness_Parse(br.ReadUInt16)
        Dim ebr As New EndianBinaryReader(br)
        br = EndianBinaryReader.DecideEndianReader(ebr, m_en)
        Data.Read(br)
        fs.Close()
    End Sub
    Private Function Endianness_Parse(s As String) As Endianness
        Return If(Strings.UCase(Strings.Left(s, 1)) = "I", Endianness.IntelLittleEndian, Endianness.MotorolaBigEndian)
    End Function
    Private Function Endianness_Parse(Value As UInt16) As Endianness
        Return If(Value = &H4949, Endianness.IntelLittleEndian, Endianness.MotorolaBigEndian)
    End Function
End Class
Public Class Data
    Public i16 As Int16   ' 2
    Public ui16 As UInt16 ' 2
    Public i32 As Int32   ' 4
    Public ui32 As UInt32 ' 4
    Public sng As Single  ' 4 
    Public i64 As Int64   ' 8
    Public ui64 As UInt64 ' 8
    Public dbl As Double  ' 8
    '                 Sum: 40
    Public Shared Function DefaultData() As Data
        Dim d As New Data
        With d
            .i16 = &H4242 : .ui16 = &H3456 : .i32 = &H12345678 : .ui32 = &H23456789 : .sng = 123456.789!
            .i64 = &H12345678ABCDEF00 : .ui64 = &H12345678ABCDEF00 : .dbl = 12345678.12345678#
        End With
        Return d
    End Function
    Public Function IsEqual(other As Data) As Boolean
        If i16 <> other.i16 Then Return False
        If ui16 <> other.ui16 Then Return False
        If i32 <> other.i32 Then Return False
        If ui32 <> other.ui32 Then Return False
        If sng <> other.sng Then Return False
        If i64 <> other.i64 Then Return False
        If ui64 <> other.ui64 Then Return False
        If dbl <> other.dbl Then Return False
        Return True
    End Function
    Public Sub Write(bw As IO.BinaryWriter)
        Using bw
            bw.Write(i16) : bw.Write(ui16) : bw.Write(i32) : bw.Write(ui32)
            bw.Write(sng) : bw.Write(i64) : bw.Write(ui64) : bw.Write(dbl)
        End Using
    End Sub
    Public Sub Read(br As IO.BinaryReader)
        Using br
            i16 = br.ReadInt16 : ui16 = br.ReadUInt16 : i32 = br.ReadInt32 : ui32 = br.ReadUInt32
            sng = br.ReadSingle : i64 = br.ReadInt64 : ui64 = br.ReadUInt64 : dbl = br.ReadDouble
        End Using
    End Sub
End Class
Public Structure B8
    Public B0 As Byte
    Public B1 As Byte
    Public B2 As Byte
    Public B3 As Byte
    Public B4 As Byte
    Public B5 As Byte
    Public B6 As Byte
    Public B7 As Byte
    Public Sub Swap_16() 'As B8
        Dim tmp As Byte
        tmp = B3 : B3 = B4 : B4 = tmp
    End Sub
    Public Sub Swap_32() 'As B8
        Dim tmp As Byte
        tmp = B3 : B3 = B4 : B4 = tmp
        tmp = B2 : B2 = B5 : B5 = tmp
    End Sub
    Public Sub Swap_64() 'As B8
        Dim tmp As Byte
        tmp = B3 : B3 = B4 : B4 = tmp
        tmp = B2 : B2 = B5 : B5 = tmp
        tmp = B1 : B1 = B6 : B6 = tmp
        tmp = B0 : B0 = B7 : B7 = tmp
    End Sub
    Public Overrides Function ToString() As String
        Dim sb As New Text.StringBuilder
        sb.Append(B0.ToString("X2")).Append(" ")
        sb.Append(B1.ToString("X2")).Append(" ")
        sb.Append(B2.ToString("X2")).Append(" ")
        sb.Append(B3.ToString("X2")).Append(" ")
        sb.Append(B4.ToString("X2")).Append(" ")
        sb.Append(B5.ToString("X2")).Append(" ")
        sb.Append(B6.ToString("X2")).Append(" ")
        sb.Append(B7.ToString("X2"))
        Return sb.ToString
    End Function
End Structure

<System.Runtime.InteropServices.StructLayout(System.Runtime.InteropServices.LayoutKind.Explicit)>
Public Structure UnionSBO
    <System.Runtime.InteropServices.FieldOffset(0)>
    Public B8 As B8

    <System.Runtime.InteropServices.FieldOffset(0)>
    Public Int64Val As Int64

    <System.Runtime.InteropServices.FieldOffset(0)>
    Public UInt64Val As UInt64

    <System.Runtime.InteropServices.FieldOffset(0)>
    Public DblVal As Double

    <System.Runtime.InteropServices.FieldOffset(2)>
    Public Int32Val As Int32
    <System.Runtime.InteropServices.FieldOffset(2)>
    Public UInt32Val As UInt32
    <System.Runtime.InteropServices.FieldOffset(2)>
    Public SngVal As Single
    <System.Runtime.InteropServices.FieldOffset(3)>
    Public Int16Val As Int16
    <System.Runtime.InteropServices.FieldOffset(3)>
    Public UInt16Val As UInt16
End Structure

Public Class EndianBinaryReader : Inherits System.IO.BinaryReader

    Private m_uni As UnionSBO
    Private ReadOnly m_br As IO.BinaryReader

    Public Sub New(br As IO.BinaryReader)
        MyBase.New(br.BaseStream)
        m_br = br
    End Sub
    Public Shared Function DecideEndianReader(ebr As EndianBinaryReader, e As Endianness) As IO.BinaryReader
        Select Case e
            Case Endianness.IntelLittleEndian : Return If(BitConverter.IsLittleEndian, ebr.BaseReader, ebr)
            Case Endianness.MotorolaBigEndian : Return If(BitConverter.IsLittleEndian, ebr, ebr.BaseReader)
            Case Else : Return Nothing
        End Select
    End Function
    Public ReadOnly Property BaseReader As IO.BinaryReader
        Get
            Return m_br
        End Get
    End Property
    Public Overrides Function ReadInt16() As Short
        m_uni.Int16Val = m_br.ReadInt16
        m_uni.B8.Swap_16()
        Return m_uni.Int16Val
    End Function
    Public Overrides Function ReadUInt16() As UInt16
        m_uni.UInt16Val = m_br.ReadUInt16
        m_uni.B8.Swap_16()
        Return m_uni.UInt16Val
    End Function
    Public Overrides Function ReadInt32() As Int32
        m_uni.Int32Val = m_br.ReadInt32
        m_uni.B8.Swap_32()
        Return m_uni.Int32Val
    End Function
    Public Overrides Function ReadUInt32() As UInt32
        m_uni.UInt32Val = m_br.ReadUInt32
        m_uni.B8.Swap_32()
        Return m_uni.UInt32Val
    End Function
    Public Overrides Function ReadSingle() As Single
        m_uni.SngVal = m_br.ReadSingle
        m_uni.B8.Swap_32()
        Return m_uni.SngVal
    End Function
    Public Overrides Function ReadInt64() As Int64
        m_uni.Int64Val = m_br.ReadInt64
        m_uni.B8.Swap_64()
        Return m_uni.Int64Val
    End Function
    Public Overrides Function ReadUInt64() As UInt64
        m_uni.UInt64Val = m_br.ReadUInt64
        m_uni.B8.Swap_64()
        Return m_uni.UInt64Val
    End Function
    Public Overrides Function ReadDouble() As Double
        m_uni.DblVal = m_br.ReadDouble
        m_uni.B8.Swap_64()
        Return m_uni.DblVal
    End Function
End Class
Public Class EndianBinaryWriter : Inherits IO.BinaryWriter

    Private m_uni As UnionSBO
    Private ReadOnly m_bw As IO.BinaryWriter

    Public Sub New(bw As IO.BinaryWriter)
        'MyBase.New(bw.BaseStream)
        m_bw = bw
    End Sub
    Public Shared Function DecideEndianWriter(ebw As EndianBinaryWriter, e As Endianness) As IO.BinaryWriter
        Dim bw As IO.BinaryWriter
        Select Case e
            Case Endianness.IntelLittleEndian : bw = If(BitConverter.IsLittleEndian, ebw.BaseWriter, ebw)
            Case Endianness.MotorolaBigEndian : bw = If(BitConverter.IsLittleEndian, ebw, ebw.BaseWriter)
            Case Else : bw = Nothing
        End Select
        'Debug.Print(bw.ToString)
        Return bw
    End Function
    Public ReadOnly Property BaseWriter As IO.BinaryWriter
        Get
            Return m_bw
        End Get
    End Property
    Public Overrides Sub Write(Value As Int16)
        m_uni.Int16Val = Value
        m_uni.B8.Swap_16()
        m_bw.Write(m_uni.Int16Val)
    End Sub
    Public Overrides Sub Write(Value As UInt16)
        m_uni.UInt16Val = Value
        m_uni.B8.Swap_16()
        m_bw.Write(m_uni.UInt16Val)
    End Sub
    Public Overrides Sub Write(Value As Int32)
        m_uni.Int32Val = Value
        m_uni.B8.Swap_32()
        m_bw.Write(m_uni.Int32Val)
    End Sub
    Public Overrides Sub Write(Value As UInt32)
        m_uni.UInt32Val = Value
        m_uni.B8.Swap_32()
        m_bw.Write(m_uni.UInt32Val)
    End Sub
    Public Overrides Sub Write(Value As Single)
        m_uni.SngVal = Value
        m_uni.B8.Swap_32()
        m_bw.Write(m_uni.SngVal)
    End Sub
    Public Overrides Sub Write(Value As Int64)
        m_uni.Int64Val = Value
        m_uni.B8.Swap_64()
        m_bw.Write(m_uni.Int64Val)
    End Sub
    Public Overrides Sub Write(Value As UInt64)
        m_uni.UInt64Val = Value
        m_uni.B8.Swap_64()
        m_bw.Write(m_uni.UInt64Val)
    End Sub
    Public Overrides Sub Write(Value As Double)
        m_uni.DblVal = Value
        m_uni.B8.Swap_64()
        m_bw.Write(m_uni.DblVal)
    End Sub
End Class